@@ -488,14 +488,15 @@ gst_element_factory_create_with_properties (GstElementFactory * factory,
 
   GST_INFO ("creating element \"%s\"", GST_OBJECT_NAME (factory));
 
-  if (factory->type == 0)
+  if (factory->type == G_TYPE_INVALID)
     goto no_type;
 
   element = (GstElement *) g_object_new_with_properties (factory->type, n,
       names, values);
-
-  if (G_UNLIKELY (element == NULL))
-    goto no_element;
+  if (G_UNLIKELY (!element)) {
+    gst_object_unref (factory);
+    g_return_val_if_fail (element != NULL, NULL);
+  }
 
   /* fill in the pointer to the factory in the element class. The
    * class will not be unreffed currently.
@@ -535,12 +536,6 @@ no_type:
     gst_object_unref (factory);
     return NULL;
   }
-no_element:
-  {
-    GST_WARNING_OBJECT (factory, "could not create element");
-    gst_object_unref (factory);
-    return NULL;
-  }
 }
 
 /**
@@ -573,11 +568,14 @@ gst_element_factory_create_valist (GstElementFactory * factory,
       GST_ELEMENT_FACTORY (gst_plugin_feature_load (GST_PLUGIN_FEATURE
           (factory)));
 
-  g_return_val_if_fail (newfactory != NULL, NULL);
-  g_return_val_if_fail (newfactory->type != 0, NULL);
+  if (newfactory == NULL)
+    goto load_failed;
 
   factory = newfactory;
 
+  if (factory->type == G_TYPE_INVALID)
+    goto no_type;
+
   if (!first) {
     element =
         gst_element_factory_create_with_properties (factory, 0, NULL, NULL);
@@ -602,6 +600,19 @@ gst_element_factory_create_valist (GstElementFactory * factory,
 out:
   gst_object_unref (factory);
   return element;
+
+  /* ERRORS */
+load_failed:
+  {
+    GST_WARNING_OBJECT (factory, "loading plugin returned NULL!");
+    return NULL;
+  }
+no_type:
+  {
+    GST_WARNING_OBJECT (factory, "factory has no type");
+    gst_object_unref (factory);
+    return NULL;
+  }
 }
 
 /**
