@@ -230,7 +230,10 @@ typedef enum _GstQuarkId
   GST_QUARK_OFFSET = 199,
   GST_QUARK_PLUGIN_API = 200,
   GST_QUARK_PLUGIN_API_FLAGS = 201,
-  GST_QUARK_MAX = 202
+  GST_QUARK_GAP_FLAGS = 202,
+  GST_QUARK_QUERY_SELECTABLE = 203,
+  GST_QUARK_SELECTABLE = 204,
+  GST_QUARK_MAX = 205
 } GstQuarkId;
 
 extern GQuark _priv_gst_quark_table[GST_QUARK_MAX];
