@@ -26,6 +26,8 @@
 #include <stdarg.h>
 #include <math.h>
 
+#include <glib.h>
+
 #include "internal-check.h"
 #include "check_error.h"
 #include "check_list.h"
@@ -386,6 +388,7 @@ _ck_assert_failed (const char *file, int line, const char *expr, ...)
   send_failure_info (to_send);
 #if defined(HAVE_FORK) && HAVE_FORK==1
   if (cur_fork_status () == CK_FORK) {
+    g_thread_pool_stop_unused_threads ();
     _exit (1);
   }
 #endif /* HAVE_FORK */
