@@ -21,16 +21,16 @@
  */
 /**
  * SECTION:element-filesrc
+ * @title: filesrc
  * @see_also: #GstFileSrc
  *
  * Read data from a file in the local file system.
  *
- * <refsect2>
- * <title>Example launch line</title>
+ * ## Example launch line
  * |[
  * gst-launch-1.0 filesrc location=song.ogg ! decodebin ! audioconvert ! audioresample ! autoaudiosink
  * ]| Play song.ogg audio file which must be in the current working directory.
- * </refsect2>
+ *
  */
 
 #ifdef HAVE_CONFIG_H
@@ -38,19 +38,18 @@
 #endif
 
 #include <gst/gst.h>
+#include <glib/gstdio.h>
 #include "gstfilesrc.h"
+#include "gstcoreelementselements.h"
 
 #include <stdio.h>
 #include <sys/types.h>
 #include <sys/stat.h>
 #ifdef G_OS_WIN32
+#include <windows.h>
 #include <io.h>                 /* lseek, open, close, read */
 /* On win32, stat* default to 32 bit; we need the 64-bit
  * variants, so explicitly define it that way. */
-#undef stat
-#define stat __stat64
-#undef fstat
-#define fstat _fstat64
 #undef lseek
 #define lseek _lseeki64
 #undef off_t
@@ -59,25 +58,28 @@
  * _stat*, since we're explicitly overriding that */
 #undef _INC_STAT_INL
 #endif
+
 #include <fcntl.h>
 
 #ifdef HAVE_UNISTD_H
 #  include <unistd.h>
 #endif
 
+#define struct_stat struct stat
+
 #ifdef __BIONIC__               /* Android */
-#undef lseek
-#define lseek lseek64
+#if defined(__ANDROID_API__) && __ANDROID_API__ >= 21
 #undef fstat
 #define fstat fstat64
-#undef off_t
-#define off_t guint64
+#undef struct_stat
+#define struct_stat struct stat64
+#endif
 #endif
 
 #include <errno.h>
 #include <string.h>
 
-#include "../../gst/gst-i18n-lib.h"
+#include <glib/gi18n-lib.h>
 
 static GstStaticPadTemplate srctemplate = GST_STATIC_PAD_TEMPLATE ("src",
     GST_PAD_SRC,
@@ -97,36 +99,6 @@ static GstStaticPadTemplate srctemplate = GST_STATIC_PAD_TEMPLATE ("src",
 #define O_BINARY (0)
 #endif
 
-/* Copy of glib's g_open due to win32 libc/cross-DLL brokenness: we can't
- * use the 'file descriptor' opened in glib (and returned from this function)
- * in this library, as they may have unrelated C runtimes. */
-static int
-gst_open (const gchar * filename, int flags, int mode)
-{
-#ifdef G_OS_WIN32
-  wchar_t *wfilename = g_utf8_to_utf16 (filename, -1, NULL, NULL, NULL);
-  int retval;
-  int save_errno;
-
-  if (wfilename == NULL) {
-    errno = EINVAL;
-    return -1;
-  }
-
-  retval = _wopen (wfilename, flags, mode);
-  save_errno = errno;
-
-  g_free (wfilename);
-
-  errno = save_errno;
-  return retval;
-#elif defined (__BIONIC__)
-  return open (filename, flags | O_LARGEFILE, mode);
-#else
-  return open (filename, flags, mode);
-#endif
-}
-
 GST_DEBUG_CATEGORY_STATIC (gst_file_src_debug);
 #define GST_CAT_DEFAULT gst_file_src_debug
 
@@ -168,6 +140,8 @@ static void gst_file_src_uri_handler_init (gpointer g_iface,
   GST_DEBUG_CATEGORY_INIT (gst_file_src_debug, "filesrc", 0, "filesrc element");
 #define gst_file_src_parent_class parent_class
 G_DEFINE_TYPE_WITH_CODE (GstFileSrc, gst_file_src, GST_TYPE_BASE_SRC, _do_init);
+GST_ELEMENT_REGISTER_DEFINE (filesrc, "filesrc", GST_RANK_PRIMARY,
+    GST_TYPE_FILE_SRC);
 
 static void
 gst_file_src_class_init (GstFileSrcClass * klass)
@@ -236,7 +210,8 @@ gst_file_src_finalize (GObject * object)
 }
 
 static gboolean
-gst_file_src_set_location (GstFileSrc * src, const gchar * location)
+gst_file_src_set_location (GstFileSrc * src, const gchar * location,
+    GError ** err)
 {
   GstState state;
 
@@ -272,6 +247,10 @@ wrong_state:
   {
     g_warning ("Changing the `location' property on filesrc when a file is "
         "open is not supported.");
+    if (err)
+      g_set_error (err, GST_URI_ERROR, GST_URI_ERROR_BAD_STATE,
+          "Changing the `location' property on filesrc when a file is "
+          "open is not supported.");
     GST_OBJECT_UNLOCK (src);
     return FALSE;
   }
@@ -289,7 +268,7 @@ gst_file_src_set_property (GObject * object, guint prop_id,
 
   switch (prop_id) {
     case PROP_LOCATION:
-      gst_file_src_set_location (src, g_value_get_string (value));
+      gst_file_src_set_location (src, g_value_get_string (value), NULL);
       break;
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
@@ -427,7 +406,6 @@ gst_file_src_is_seekable (GstBaseSrc * basesrc)
 static gboolean
 gst_file_src_get_size (GstBaseSrc * basesrc, guint64 * size)
 {
-  struct stat stat_results;
   GstFileSrc *src;
 
   src = GST_FILE_SRC (basesrc);
@@ -437,11 +415,30 @@ gst_file_src_get_size (GstBaseSrc * basesrc, guint64 * size)
      * succeed, and wrongly say our length is zero. */
     return FALSE;
   }
+#ifdef G_OS_WIN32
+  {
+    HANDLE h = (HANDLE) _get_osfhandle (src->fd);
+    LARGE_INTEGER file_size;
 
-  if (fstat (src->fd, &stat_results) < 0)
-    goto could_not_stat;
+    if (h == INVALID_HANDLE_VALUE)
+      goto could_not_stat;
 
-  *size = stat_results.st_size;
+    if (!GetFileSizeEx (h, &file_size)) {
+      goto could_not_stat;
+    }
+
+    *size = file_size.QuadPart;
+  }
+#else
+  {
+    struct_stat stat_results;
+
+    if (fstat (src->fd, &stat_results) < 0)
+      goto could_not_stat;
+
+    *size = stat_results.st_size;
+  }
+#endif
 
   return TRUE;
 
@@ -457,7 +454,10 @@ static gboolean
 gst_file_src_start (GstBaseSrc * basesrc)
 {
   GstFileSrc *src = GST_FILE_SRC (basesrc);
-  struct stat stat_results;
+  int flags = O_RDONLY | O_BINARY;
+#if defined (__BIONIC__)
+  flags |= O_LARGEFILE;
+#endif
 
   if (src->filename == NULL || src->filename[0] == '\0')
     goto no_filename;
@@ -465,27 +465,51 @@ gst_file_src_start (GstBaseSrc * basesrc)
   GST_INFO_OBJECT (src, "opening file %s", src->filename);
 
   /* open the file */
-  src->fd = gst_open (src->filename, O_RDONLY | O_BINARY, 0);
+  src->fd = g_open (src->filename, flags, 0);
 
   if (src->fd < 0)
     goto open_failed;
 
-  /* check if it is a regular file, otherwise bail out */
-  if (fstat (src->fd, &stat_results) < 0)
-    goto no_stat;
+#ifdef G_OS_WIN32
+  {
+    HANDLE h = (HANDLE) _get_osfhandle (src->fd);
+    FILE_STANDARD_INFO file_info;
 
-  if (S_ISDIR (stat_results.st_mode))
-    goto was_directory;
+    if (h == INVALID_HANDLE_VALUE)
+      goto no_stat;
 
-  if (S_ISSOCK (stat_results.st_mode))
-    goto was_socket;
+    if (!GetFileInformationByHandleEx (h, FileStandardInfo, &file_info,
+            sizeof (FILE_STANDARD_INFO)))
+      goto no_stat;
+
+    if (file_info.Directory)
+      goto was_directory;
+
+    /* everything's a regular file on Windows */
+    src->is_regular = TRUE;
+  }
+#else
+  {
+    struct_stat stat_results;
+
+    /* check if it is a regular file, otherwise bail out */
+    if (fstat (src->fd, &stat_results) < 0)
+      goto no_stat;
+
+    if (S_ISDIR (stat_results.st_mode))
+      goto was_directory;
+
+    if (S_ISSOCK (stat_results.st_mode))
+      goto was_socket;
+
+    /* record if it's a regular (hence seekable and lengthable) file */
+    if (S_ISREG (stat_results.st_mode))
+      src->is_regular = TRUE;
+  }
+#endif
 
   src->read_position = 0;
 
-  /* record if it's a regular (hence seekable and lengthable) file */
-  if (S_ISREG (stat_results.st_mode))
-    src->is_regular = TRUE;
-
   /* We need to check if the underlying file is seekable. */
   {
     off_t res = lseek (src->fd, 0, SEEK_END);
@@ -549,12 +573,14 @@ was_directory:
         (_("\"%s\" is a directory."), src->filename), (NULL));
     goto error_close;
   }
+#ifndef G_OS_WIN32
 was_socket:
   {
     GST_ELEMENT_ERROR (src, RESOURCE, OPEN_READ,
         (_("File \"%s\" is a socket."), src->filename), (NULL));
     goto error_close;
   }
+#endif
 lseek_wonky:
   {
     GST_ELEMENT_ERROR (src, RESOURCE, OPEN_READ, (NULL),
@@ -575,7 +601,7 @@ gst_file_src_stop (GstBaseSrc * basesrc)
   GstFileSrc *src = GST_FILE_SRC (basesrc);
 
   /* close the file */
-  close (src->fd);
+  g_close (src->fd, NULL);
 
   /* zero out a lot of our state */
   src->fd = 0;
@@ -621,7 +647,7 @@ gst_file_src_uri_set_uri (GstURIHandler * handler, const gchar * uri,
     /* Special case for "file://" as this is used by some applications
      *  to test with gst_element_make_from_uri if there's an element
      *  that supports the URI protocol. */
-    gst_file_src_set_location (src, NULL);
+    gst_file_src_set_location (src, NULL, NULL);
     return TRUE;
   }
 
@@ -643,14 +669,14 @@ gst_file_src_uri_set_uri (GstURIHandler * handler, const gchar * uri,
 #ifdef G_OS_WIN32
   /* Unfortunately, g_filename_from_uri() doesn't handle some UNC paths
    * correctly on windows, it leaves them with an extra backslash
-   * at the start if they're of the mozilla-style file://///host/path/file 
+   * at the start if they're of the mozilla-style file://///host/path/file
    * form. Correct this.
    */
   if (location[0] == '\\' && location[1] == '\\' && location[2] == '\\')
     memmove (location, location + 1, strlen (location + 1) + 1);
 #endif
 
-  ret = gst_file_src_set_location (src, location);
+  ret = gst_file_src_set_location (src, location, err);
 
 beach:
   if (location)
