@@ -36,8 +36,9 @@
 #include <string.h>
 
 #include "gstelements_private.h"
-#include "../../gst/gst-i18n-lib.h"
+#include <glib/gi18n-lib.h>
 #include "gstidentity.h"
+#include "gstcoreelementselements.h"
 
 static GstStaticPadTemplate sinktemplate = GST_STATIC_PAD_TEMPLATE ("sink",
     GST_PAD_SINK,
@@ -95,7 +96,8 @@ enum
   PROP_CHECK_IMPERFECT_OFFSET,
   PROP_SIGNAL_HANDOFFS,
   PROP_DROP_ALLOCATION,
-  PROP_EOS_AFTER
+  PROP_EOS_AFTER,
+  PROP_STATS
 };
 
 
@@ -104,6 +106,8 @@ enum
 #define gst_identity_parent_class parent_class
 G_DEFINE_TYPE_WITH_CODE (GstIdentity, gst_identity, GST_TYPE_BASE_TRANSFORM,
     _do_init);
+GST_ELEMENT_REGISTER_DEFINE (identity, "identity", GST_RANK_NONE,
+    GST_TYPE_IDENTITY);
 
 static void gst_identity_finalize (GObject * object);
 static void gst_identity_set_property (GObject * object, guint prop_id,
@@ -113,6 +117,8 @@ static void gst_identity_get_property (GObject * object, guint prop_id,
 
 static gboolean gst_identity_sink_event (GstBaseTransform * trans,
     GstEvent * event);
+static gboolean gst_identity_src_event (GstBaseTransform * trans,
+    GstEvent * event);
 static GstFlowReturn gst_identity_transform_ip (GstBaseTransform * trans,
     GstBuffer * buf);
 static gboolean gst_identity_start (GstBaseTransform * trans);
@@ -123,6 +129,7 @@ static gboolean gst_identity_accept_caps (GstBaseTransform * base,
     GstPadDirection direction, GstCaps * caps);
 static gboolean gst_identity_query (GstBaseTransform * base,
     GstPadDirection direction, GstQuery * query);
+static GstClock *gst_identity_provide_clock (GstElement * element);
 
 static guint gst_identity_signals[LAST_SIGNAL] = { 0 };
 
@@ -255,15 +262,43 @@ gst_identity_class_init (GstIdentityClass * klass)
    * GstIdentity::handoff:
    * @identity: the identity instance
    * @buffer: the buffer that just has been received
-   * @pad: the pad that received it
    *
    * This signal gets emitted before passing the buffer downstream.
    */
   gst_identity_signals[SIGNAL_HANDOFF] =
       g_signal_new ("handoff", G_TYPE_FROM_CLASS (klass), G_SIGNAL_RUN_LAST,
       G_STRUCT_OFFSET (GstIdentityClass, handoff), NULL, NULL,
-      g_cclosure_marshal_generic, G_TYPE_NONE, 1,
-      GST_TYPE_BUFFER | G_SIGNAL_TYPE_STATIC_SCOPE);
+      NULL, G_TYPE_NONE, 1, GST_TYPE_BUFFER | G_SIGNAL_TYPE_STATIC_SCOPE);
+
+  /**
+   * GstIdentity:stats:
+
+   * Various statistics. This property returns a GstStructure
+   * with name application/x-identity-stats with the following fields:
+   *
+   * <itemizedlist>
+   * <listitem>
+   *   <para>
+   *   #guint64
+   *   <classname>&quot;num-buffers&quot;</classname>:
+   *   the number of buffers that passed through.
+   *   </para>
+   * </listitem>
+   * <listitem>
+   *   <para>
+   *   #guint64
+   *   <classname>&quot;num-bytes&quot;</classname>:
+   *   the number of bytes that passed through.
+   *   </para>
+   * </listitem>
+   * </itemizedlist>
+   *
+   * Since: 1.20
+   */
+  g_object_class_install_property (gobject_class, PROP_STATS,
+      g_param_spec_boxed ("stats", "Statistics",
+          "Statistics", GST_TYPE_STRUCTURE,
+          G_PARAM_READABLE | G_PARAM_STATIC_STRINGS));
 
   gobject_class->finalize = gst_identity_finalize;
 
@@ -276,8 +311,11 @@ gst_identity_class_init (GstIdentityClass * klass)
 
   gstelement_class->change_state =
       GST_DEBUG_FUNCPTR (gst_identity_change_state);
+  gstelement_class->provide_clock =
+      GST_DEBUG_FUNCPTR (gst_identity_provide_clock);
 
   gstbasetrans_class->sink_event = GST_DEBUG_FUNCPTR (gst_identity_sink_event);
+  gstbasetrans_class->src_event = GST_DEBUG_FUNCPTR (gst_identity_src_event);
   gstbasetrans_class->transform_ip =
       GST_DEBUG_FUNCPTR (gst_identity_transform_ip);
   gstbasetrans_class->start = GST_DEBUG_FUNCPTR (gst_identity_start);
@@ -310,6 +348,8 @@ gst_identity_init (GstIdentity * identity)
   identity->eos_after_counter = DEFAULT_EOS_AFTER;
 
   gst_base_transform_set_gap_aware (GST_BASE_TRANSFORM_CAST (identity), TRUE);
+
+  GST_OBJECT_FLAG_SET (identity, GST_ELEMENT_FLAG_REQUIRE_CLOCK);
 }
 
 static void
@@ -421,7 +461,14 @@ gst_identity_sink_event (GstBaseTransform * trans, GstEvent * event)
 
       /* This is the first segment, send out a (0, -1) segment */
       gst_segment_init (&segment, segment.format);
+      if (identity->seek_segment.format != GST_FORMAT_UNDEFINED) {
+        segment.time = identity->seek_segment.time;
+        segment.base = identity->seek_segment.base;
+        gst_segment_init (&identity->seek_segment, GST_FORMAT_UNDEFINED);
+      }
+
       news = gst_event_new_segment (&segment);
+      GST_EVENT_SEQNUM (news) = GST_EVENT_SEQNUM (event);
 
       gst_pad_event_default (trans->sinkpad, GST_OBJECT_CAST (trans), news);
     } else {
@@ -432,19 +479,34 @@ gst_identity_sink_event (GstBaseTransform * trans, GstEvent * event)
 
   if (GST_EVENT_TYPE (event) == GST_EVENT_GAP &&
       trans->have_segment && trans->segment.format == GST_FORMAT_TIME) {
-    GstClockTime start, dur;
+    GstClockTime start, dur, old_start;
 
-    gst_event_parse_gap (event, &start, &dur);
-    if (GST_CLOCK_TIME_IS_VALID (start)) {
-      start = gst_segment_to_running_time (&trans->segment,
-          GST_FORMAT_TIME, start);
+    gst_event_parse_gap (event, &old_start, &dur);
 
-      gst_identity_do_sync (identity, start);
+    start = gst_segment_to_running_time (&trans->segment,
+        GST_FORMAT_TIME, old_start);
 
-      /* also transform GAP timestamp similar to buffer timestamps */
-      if (identity->single_segment) {
+    gst_identity_do_sync (identity,
+        GST_CLOCK_TIME_IS_VALID (start) ? start : 0);
+
+    /* also transform GAP timestamp similar to buffer timestamps */
+    if (identity->single_segment) {
+      guint64 clip_start, clip_stop;
+      if (GST_CLOCK_TIME_IS_VALID (start)) {
         gst_event_unref (event);
         event = gst_event_new_gap (start, dur);
+      } else if (GST_CLOCK_TIME_IS_VALID (dur) &&
+          gst_segment_clip (&trans->segment, GST_FORMAT_TIME, old_start,
+              old_start + dur, &clip_start, &clip_stop)) {
+        gst_event_unref (event);
+        event = gst_event_new_gap (clip_start, clip_stop - clip_start);
+      } else {
+        /* Gap event is completely outside the segment, drop it */
+        GST_DEBUG_OBJECT (identity,
+            "Dropping GAP event outside segment: %" GST_PTR_FORMAT, event);
+        gst_event_unref (event);
+        ret = TRUE;
+        goto done;
       }
     }
   }
@@ -460,8 +522,11 @@ gst_identity_sink_event (GstBaseTransform * trans, GstEvent * event)
     /* eat up segments */
     gst_event_unref (event);
     ret = TRUE;
-  } else {
-    if (GST_EVENT_TYPE (event) == GST_EVENT_FLUSH_START) {
+    goto done;
+  }
+
+  switch (GST_EVENT_TYPE (event)) {
+    case GST_EVENT_FLUSH_START:
       GST_OBJECT_LOCK (identity);
       identity->flushing = TRUE;
       g_cond_signal (&identity->blocked_cond);
@@ -470,18 +535,60 @@ gst_identity_sink_event (GstBaseTransform * trans, GstEvent * event)
         gst_clock_id_unschedule (identity->clock_id);
       }
       GST_OBJECT_UNLOCK (identity);
-    } else if (GST_EVENT_TYPE (event) == GST_EVENT_FLUSH_STOP) {
+      break;
+    case GST_EVENT_FLUSH_STOP:
       GST_OBJECT_LOCK (identity);
       identity->flushing = FALSE;
+      trans->have_segment = FALSE;
       GST_OBJECT_UNLOCK (identity);
-    }
-
-    ret = GST_BASE_TRANSFORM_CLASS (parent_class)->sink_event (trans, event);
+      break;
+    default:
+      break;
   }
 
+  ret = GST_BASE_TRANSFORM_CLASS (parent_class)->sink_event (trans, event);
+
+done:
   return ret;
 }
 
+static gboolean
+gst_identity_src_event (GstBaseTransform * trans, GstEvent * event)
+{
+  GstIdentity *identity = GST_IDENTITY (trans);
+
+  switch (GST_EVENT_TYPE (event)) {
+    case GST_EVENT_SEEK:
+    {
+      gdouble rate;
+      GstFormat fmt;
+      GstSeekFlags flags;
+      GstSeekType start_type, stop_type;
+      gint64 start, stop;
+      gst_event_parse_seek (event, &rate, &fmt, &flags, &start_type,
+          &start, &stop_type, &stop);
+
+      GST_OBJECT_LOCK (identity);
+      gst_segment_init (&identity->seek_segment, fmt);
+      if (!gst_segment_do_seek (&identity->seek_segment, rate, fmt,
+              flags, start_type, start, stop_type, stop, NULL)) {
+        GST_WARNING_OBJECT (identity, "Could not run seek %" GST_PTR_FORMAT,
+            event);
+        GST_OBJECT_UNLOCK (identity);
+
+        return FALSE;
+      }
+      GST_OBJECT_UNLOCK (identity);
+
+      break;
+    }
+    default:
+      break;
+  }
+
+  return GST_BASE_TRANSFORM_CLASS (parent_class)->src_event (trans, event);
+}
+
 static void
 gst_identity_check_imperfect_timestamp (GstIdentity * identity, GstBuffer * buf)
 {
@@ -614,6 +721,7 @@ gst_identity_update_last_message_for_buffer (GstIdentity * identity,
       GST_BUFFER_FLAGS (buf), flag_str, meta_str ? meta_str : "none", buf);
   g_free (flag_str);
   g_free (meta_str);
+  GST_TRACE_OBJECT (identity, "%s", identity->last_message);
 
   GST_OBJECT_UNLOCK (identity);
 
@@ -688,10 +796,21 @@ gst_identity_transform_ip (GstBaseTransform * trans, GstBuffer * buf)
     g_signal_emit (identity, gst_identity_signals[SIGNAL_HANDOFF], 0, buf);
 
   if (trans->segment.format == GST_FORMAT_TIME) {
-    rundts = gst_segment_to_running_time (&trans->segment,
-        GST_FORMAT_TIME, GST_BUFFER_DTS (buf));
-    runpts = gst_segment_to_running_time (&trans->segment,
-        GST_FORMAT_TIME, GST_BUFFER_PTS (buf));
+    if (trans->segment.rate > 0) {
+      runpts = gst_segment_to_running_time (&trans->segment,
+          GST_FORMAT_TIME, GST_BUFFER_PTS (buf));
+      rundts = gst_segment_to_running_time (&trans->segment,
+          GST_FORMAT_TIME, GST_BUFFER_DTS (buf));
+    } else {
+      runpts = gst_segment_to_running_time (&trans->segment,
+          GST_FORMAT_TIME, GST_CLOCK_TIME_IS_VALID (buf->duration)
+          && GST_CLOCK_TIME_IS_VALID (buf->pts) ? buf->pts +
+          buf->duration : buf->pts);
+      rundts = gst_segment_to_running_time (&trans->segment,
+          GST_FORMAT_TIME, GST_CLOCK_TIME_IS_VALID (buf->duration)
+          && GST_CLOCK_TIME_IS_VALID (buf->dts) ? buf->dts +
+          buf->duration : buf->dts);
+    }
   }
 
   if (GST_CLOCK_TIME_IS_VALID (rundts))
@@ -715,6 +834,11 @@ gst_identity_transform_ip (GstBaseTransform * trans, GstBuffer * buf)
     GST_BUFFER_OFFSET_END (buf) = GST_CLOCK_TIME_NONE;
   }
 
+  GST_OBJECT_LOCK (trans);
+  identity->num_bytes += gst_buffer_get_size (buf);
+  identity->num_buffers++;
+  GST_OBJECT_UNLOCK (trans);
+
   return ret;
 
   /* ERRORS */
@@ -753,6 +877,8 @@ gst_identity_set_property (GObject * object, guint prop_id,
     const GValue * value, GParamSpec * pspec)
 {
   GstIdentity *identity;
+  GstMessage *clock_message = NULL;
+  gboolean sync;
 
   identity = GST_IDENTITY (object);
 
@@ -782,7 +908,25 @@ gst_identity_set_property (GObject * object, guint prop_id,
       identity->datarate = g_value_get_int (value);
       break;
     case PROP_SYNC:
-      identity->sync = g_value_get_boolean (value);
+      sync = g_value_get_boolean (value);
+      GST_OBJECT_LOCK (identity);
+      if (sync != identity->sync) {
+        identity->sync = sync;
+        if (sync) {
+          GST_OBJECT_FLAG_SET (identity, GST_ELEMENT_FLAG_PROVIDE_CLOCK);
+          clock_message =
+              gst_message_new_clock_provide (GST_OBJECT_CAST (identity),
+              gst_system_clock_obtain (), TRUE);
+        } else {
+          GST_OBJECT_FLAG_UNSET (identity, GST_ELEMENT_FLAG_PROVIDE_CLOCK);
+          clock_message =
+              gst_message_new_clock_lost (GST_OBJECT_CAST (identity),
+              gst_system_clock_obtain ());
+        }
+      }
+      GST_OBJECT_UNLOCK (identity);
+      if (clock_message)
+        gst_element_post_message (GST_ELEMENT_CAST (identity), clock_message);
       break;
     case PROP_TS_OFFSET:
       identity->ts_offset = g_value_get_int64 (value);
@@ -812,6 +956,20 @@ gst_identity_set_property (GObject * object, guint prop_id,
     gst_base_transform_set_passthrough (GST_BASE_TRANSFORM (identity), TRUE);
 }
 
+static GstStructure *
+gst_identity_create_stats (GstIdentity * identity)
+{
+  GstStructure *s;
+
+  GST_OBJECT_LOCK (identity);
+  s = gst_structure_new ("application/x-identity-stats",
+      "num-bytes", G_TYPE_UINT64, identity->num_bytes,
+      "num-buffers", G_TYPE_UINT64, identity->num_buffers, NULL);
+  GST_OBJECT_UNLOCK (identity);
+
+  return s;
+}
+
 static void
 gst_identity_get_property (GObject * object, guint prop_id, GValue * value,
     GParamSpec * pspec)
@@ -871,6 +1029,9 @@ gst_identity_get_property (GObject * object, guint prop_id, GValue * value,
     case PROP_EOS_AFTER:
       g_value_set_int (value, identity->eos_after);
       break;
+    case PROP_STATS:
+      g_value_take_boxed (value, gst_identity_create_stats (identity));
+      break;
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
       break;
@@ -1009,6 +1170,8 @@ gst_identity_change_state (GstElement * element, GstStateChange transition)
       GST_OBJECT_UNLOCK (identity);
       if (identity->sync)
         no_preroll = TRUE;
+      identity->num_bytes = 0;
+      identity->num_buffers = 0;
       break;
     case GST_STATE_CHANGE_PAUSED_TO_PLAYING:
       GST_OBJECT_LOCK (identity);
@@ -1055,3 +1218,15 @@ gst_identity_change_state (GstElement * element, GstStateChange transition)
 
   return ret;
 }
+
+/* FIXME: GStreamer 2.0 */
+static GstClock *
+gst_identity_provide_clock (GstElement * element)
+{
+  GstIdentity *identity = GST_IDENTITY (element);
+
+  if (!identity->sync)
+    return NULL;
+
+  return gst_system_clock_obtain ();
+}
